#define MOZ_SOURCE_STAMP 7d1f3450acc47025876964c1eca854ae027934f3
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/7d1f3450acc47025876964c1eca854ae027934f3
